import unittest

from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei, get_native_balance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules import Horizondex
from tests.core.modules import test_account, init_set_test_config


class TestHorizondex(unittest.TestCase):

    def test_shouldSwapHorizondex(self, chains=['BASE', 'LINEA']):
        init_set_test_config()

        logger.info("Test horizondex")

        for chain in chains:
            chain_instance = get_chain_instance(chain)
            web3 = init_web3(chain_instance, None)

            native_balance = get_native_balance(test_account, web3, chain_instance)
            logger.info(f"Native balance: {native_balance}")

            eth_token = Erc20Token(chain, 'ETH', web3)
            usdc_token = Erc20Token(chain, 'USDC', web3)

            Horizondex(chain_instance, web3).swap(
                NativeBalance(from_eth_to_wei(1), 'BASE', 'ETH'),
                eth_token,
                usdc_token,
                0.98,
                test_account
            )
