import unittest

from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei, get_native_balance
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules import Maverick
from tests.core.modules import init_set_test_config, test_account


class TestMaverick(unittest.TestCase):
    init_set_test_config()

    def test_maverickShouldSwap(self, chain='BASE', account=test_account, token='USDC'):
        logger.info("Test maverick")

        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, None)
        native_balance = get_native_balance(account, web3, chain_instance)
        logger.info(f"{native_balance}")

        native_balance_to_sell = NativeBalance(from_eth_to_wei(1), chain, 'ETH')

        maverick = Maverick(chain_instance, web3, '0x06e6736ca9e922766279a22b75a600fe8b8473b6')
        maverick.swap(native_balance_to_sell, 'ETH', token, 0.01, account)
        amount_to_swap = Erc20Token(chain_instance['chain'], token, web3).balance(account)
        logger.info(f"{native_balance_to_sell} > {amount_to_swap}")
        maverick.swap(amount_to_swap, token, 'ETH', 0.95, account)
