import unittest

from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules.swap.mute import Mute
from tests.core.modules import zksync_test_account, init_set_test_config


class TestMute(unittest.TestCase):
    init_set_test_config()

    def test_muteShouldSwap(self, chain='ZKSYNC', account=zksync_test_account, token='USDC'):
        logger.info("Test mute")

        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, None)

        native_balance_to_sell = NativeBalance(from_eth_to_wei(1), chain, 'ETH')

        mute = Mute(chain_instance, web3)
        mute.swap(native_balance_to_sell, 'ETH', token, 0.01, account)
        amount_to_swap = Erc20Token(chain_instance['chain'], token, web3).balance(account)
        logger.info(f"{native_balance_to_sell} > {amount_to_swap}")
        mute.swap(amount_to_swap, token, 'ETH', 0.01, account)
