import unittest

from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules import Syncswap
from tests.core.modules import zksync_test_account, init_set_test_config, test_account


class TestSyncswap(unittest.TestCase):
    init_set_test_config()

    def test_syncswapShouldSwap(self, chain='ZKSYNC', account=zksync_test_account):
        logger.info("Test syncswap")

        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, None)

        native_balance = NativeBalance(from_eth_to_wei(1), chain, 'ETH')

        syncswap = Syncswap(chain_instance, web3)
        syncswap.swap(native_balance, 'ETH', 'USDC', 0.98, account)
        amount_to_swap = Erc20Token(chain, 'USDC', web3).balance(account)
        syncswap.swap(amount_to_swap, 'USDC', 'ETH', 0.98, account)

        syncswap = Syncswap(chain_instance, web3)
        syncswap.swap(native_balance, 'ETH', 'USDT', 0.98, account)
        amount_to_swap = Erc20Token(chain, 'USDT', web3).balance(account)
        syncswap.swap(amount_to_swap, 'USDT', 'ETH', 0.98, account)

        syncswap = Syncswap(chain_instance, web3)
        syncswap.swap(native_balance, 'ETH', 'WBTC', 0.98, account)
        amount_to_swap = Erc20Token(chain, 'WBTC', web3).balance(account)
        syncswap.swap(amount_to_swap, 'WBTC', 'ETH', 0.98, account)

    def test_syncswapShouldSwapForLinea(self, chain='LINEA', account=test_account):
        logger.info("Test syncswap")

        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, None)

        native_balance = NativeBalance(from_eth_to_wei(0.1), chain, 'ETH')

        syncswap = Syncswap(chain_instance, web3)
        syncswap.swap(native_balance, 'ETH', 'USDC', 0.98, account)
        amount_to_swap = Erc20Token(chain, 'USDC', web3).balance(account)
        syncswap.swap(amount_to_swap, 'USDC', 'ETH', 0.98, account)

        syncswap = Syncswap(chain_instance, web3, {
            "ETH>USDC": "0x5Ec5b1E9b1Bd5198343ABB6E55Fb695d2F7Bb308",
            "USDC>USDT": "0x258d5f860B11ec73Ee200eB14f1b60A3B7A536a2"
        })
        syncswap.swap(native_balance, 'ETH', 'USDT', 0.98, account)
        amount_to_swap = Erc20Token(chain, 'USDT', web3).balance(account)
        syncswap.swap(amount_to_swap, 'USDT', 'ETH', 0.98, account)
