import unittest

from loguru import logger
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.domain.balance.balance import NativeBalance
from sybil_engine.domain.balance.balance_utils import from_eth_to_wei
from sybil_engine.domain.balance.tokens import Erc20Token
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules import Velocore
from tests.core.modules import local_evm_test_account, init_set_test_config


class TestVelocore(unittest.TestCase):
    init_set_test_config()

    def test_velocoreShouldSwap(self, chain='ZKSYNC'):
        logger.info("Test velocore")

        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, None)

        velocore = Velocore(chain_instance, web3)

        native_balance_to_swap = NativeBalance(from_eth_to_wei(1), chain, 'ETH')

        velocore.swap(native_balance_to_swap, 'ETH', 'USDC', 0.98, local_evm_test_account)
        amount_to_swap = Erc20Token(chain_instance['chain'], 'USDC', web3).balance(local_evm_test_account)
        velocore.swap(amount_to_swap, 'USDC', 'ETH', 0.98, local_evm_test_account)

        velocore.swap(native_balance_to_swap, 'ETH', 'WBTC', 0.98, local_evm_test_account)
        amount_to_swap = Erc20Token(chain_instance['chain'], 'WBTC', web3).balance(local_evm_test_account)
        velocore.swap(amount_to_swap, 'WBTC', 'ETH', 0.98, local_evm_test_account)
