import unittest

from loguru import logger
from sybil_engine.domain.balance.balance import NotEnoughERC20Balance

from web3_wizzard_lib.core.modules import ConcreteSwap
from tests.core.modules.__init__ import zksync_min_native_balance, init_set_test_config, \
    local_evm_test_account


def create_swap_config(app, chain, from_token, to_token, amount_interval={'from': 1, 'to': 1}):
    return {
        'chain': chain,
        'app': app,
        'from_token': from_token,
        'to_token': to_token,
        'amount_interval': amount_interval
    }


class TestConcreteSwapModule(unittest.TestCase):

    def test_syncswapShouldSwapInZKSync(self, chain='ZKSYNC', app='syncswap'):
        logger.info("Test module [CONCRETE_SWAP, 3]")

        init_set_test_config()

        concrete_swap = ConcreteSwap(zksync_min_native_balance, local_evm_test_account)

        test_config_buy_usdc = create_swap_config(app, chain, 'ETH', 'USDC')
        test_config_sell_usdc = create_swap_config(app, chain, 'USDC', 'ETH', 'all_balance')
        test_config_buy_usdt = create_swap_config(app, chain, 'ETH', 'USDT')
        test_config_sell_usdt = create_swap_config(app, chain, 'USDT', 'ETH', 'all_balance')
        test_config_buy_wbtc = create_swap_config(app, chain, 'ETH', 'WBTC')
        test_config_sell_wbtc = create_swap_config(app, chain, 'WBTC', 'ETH', 'all_balance')

        try:
            concrete_swap.execute(*concrete_swap.parse_params(test_config_buy_usdc), local_evm_test_account)
            concrete_swap.execute(*concrete_swap.parse_params(test_config_sell_usdc), local_evm_test_account)
            concrete_swap.execute(*concrete_swap.parse_params(test_config_buy_usdt), local_evm_test_account)
            concrete_swap.execute(*concrete_swap.parse_params(test_config_sell_usdt), local_evm_test_account)
            concrete_swap.execute(*concrete_swap.parse_params(test_config_buy_wbtc), local_evm_test_account)
            concrete_swap.execute(*concrete_swap.parse_params(test_config_sell_wbtc), local_evm_test_account)
        except Exception as e:
            self.fail(f"Some function raised an exception: {e}")

    def test_shouldThrowNotEnoughERC20Exception(self):
        logger.info("Test module [CONCRETE_SWAP, 3]")

        init_set_test_config()

        swap_test_config = create_swap_config('syncswap', 'ZKSYNC', 'USDC', 'ETH', 'all_balance')

        concrete_swap = ConcreteSwap(zksync_min_native_balance, local_evm_test_account)

        with self.assertRaises(NotEnoughERC20Balance):
            concrete_swap.execute(*concrete_swap.parse_params(swap_test_config), local_evm_test_account)





