import unittest

from loguru import logger
from sybil_engine.domain.account_storage import AccountStorage
from sybil_engine.utils.utils import ConfigurationException

from web3_wizzard_lib.core.modules import Dmail
from tests.core.modules.__init__ import init_set_test_config, test_min_native_balance, local_evm_test_account


class TestDmailModule(unittest.TestCase):
    init_set_test_config()

    def test_shouldSendDmail(self, chains=['LINEA', 'SCROLL']):
        logger.info("Test module [DMAIL, 5]")

        for chain in chains:
            send_dmail_test_config = {
                'chain': chain,
                'email_amount': {'from': 3, 'to': 4}
            }

            try:
                dmail = Dmail(test_min_native_balance, AccountStorage(), range(3))

                dmail.execute(*dmail.parse_params(send_dmail_test_config), local_evm_test_account)
            except Exception as e:
                self.fail(f"Some function raised an exception: {e}")

    def test_shouldThrowConfigurationExceptionOnUnsupportedChain(self, chains=['ARBITRUM']):
        logger.info("Test module [DMAIL, 5]")

        for chain in chains:
            send_dmail_test_config = {
                'chain': chain,
                'email_amount': {'from': 3, 'to': 4}
            }
            dmail = Dmail(test_min_native_balance, AccountStorage(), range(3))

            with self.assertRaises(ConfigurationException):
                dmail.parse_params(send_dmail_test_config)
