import unittest

from loguru import logger
from sybil_engine.domain.account_storage import AccountStorage

from web3_wizzard_lib.core.modules import Lending, EmptyDepositException
from tests.core.modules.__init__ import local_evm_test_account, zksync_min_native_balance, init_set_test_config

withdraw_interval = {'from': 60 * 15, 'to': 60 * 60}


class TestLending(unittest.TestCase):
    init_set_test_config()

    def test_shouldDepositAndWithdraw(self):
        logger.info("Test module [LENDING]")

        lending_deposit_test_config = {
            'amount': {'from': 0.001, 'to': 0.001},
            'action': 'DEPOSIT',
            'lending_apps': ['ERALEND'],
            'withdraw_sleep_interval': withdraw_interval
        }

        lending_withdraw_test_config = {
            'amount': {'from': 0.001, 'to': 0.001},
            'action': 'WITHDRAW',
            'lending_apps': ['ERALEND'],
            'withdraw_sleep_interval': withdraw_interval
        }

        lending = Lending(zksync_min_native_balance, AccountStorage(), 1)

        lending.execute(*lending.parse_params(lending_deposit_test_config), local_evm_test_account)
        lending.execute(*lending.parse_params(lending_withdraw_test_config), local_evm_test_account)

    def test_shouldDepositAndWithdrawAll(self):
        logger.info("Test module [LENDING]")

        lending_deposit_test_config_eralend = {
            'amount': {'from': 0.001, 'to': 0.001},
            'action': 'DEPOSIT',
            'lending_apps': ['ERALEND'],
            'withdraw_sleep_interval': withdraw_interval
        }
        lending_deposit_test_config_fusion = {
            'amount': {'from': 0.001, 'to': 0.001},
            'action': 'DEPOSIT',
            'lending_apps': ['REACTORFUSION_LANDING'],
            'withdraw_sleep_interval': withdraw_interval
        }
        lending_deposit_test_config_basilisk = {
            'amount': {'from': 0.001, 'to': 0.001},
            'action': 'DEPOSIT',
            'lending_apps': ['BASILISK_LANDING'],
            'withdraw_sleep_interval': withdraw_interval
        }

        lending_withdraw_test_config = {
            'amount': {'from': 0.001, 'to': 0.001},
            'action': 'WITHDRAW_ALL',
            'lending_apps': ['ERALEND'],
            'withdraw_sleep_interval': withdraw_interval
        }

        lending = Lending(zksync_min_native_balance, AccountStorage(), 1)

        lending.execute(*lending.parse_params(lending_deposit_test_config_basilisk), local_evm_test_account)
        lending.execute(*lending.parse_params(lending_deposit_test_config_eralend), local_evm_test_account)
        lending.execute(*lending.parse_params(lending_deposit_test_config_fusion), local_evm_test_account)

        lending.execute(*lending.parse_params(lending_withdraw_test_config), local_evm_test_account)

    def test_shouldFailToWithdraw0(self):
        logger.info("Test module [LENDING]")

        lending_withdraw_test_config = {
            'amount': {'from': 0.001, 'to': 0.001},
            'action': 'WITHDRAW',
            'lending_apps': ['ERALEND'],
            'withdraw_sleep_interval': withdraw_interval
        }

        lending = Lending(zksync_min_native_balance, AccountStorage(), 1)

        with self.assertRaises(EmptyDepositException):
            lending.execute(*lending.parse_params(lending_withdraw_test_config), local_evm_test_account)

    def test_shouldDepositAndWithdrawAll_balance(self, apps=['ERALEND', 'REACTORFUSION_LANDING', 'BASILISK_LANDING']):
        logger.info("Test module [LENDING]")

        for app in apps:
            lending_deposit_test_config = {
                'amount': 'all_balance',
                'action': 'DEPOSIT',
                'lending_apps': [app],
                'withdraw_sleep_interval': withdraw_interval
            }
            lending_withdraw_test_config = {
                'amount': 'all_balance',
                'action': 'WITHDRAW',
                'lending_apps': [app],
                'withdraw_sleep_interval': withdraw_interval
            }

            lending = Lending(zksync_min_native_balance, AccountStorage(), 1)
            lending.execute(*lending.parse_params(lending_deposit_test_config), local_evm_test_account)
            lending.execute(*lending.parse_params(lending_withdraw_test_config), local_evm_test_account)

    def test_shouldDeposit_WithdrawAll_balance(self, apps=['ERALEND', 'REACTORFUSION_LANDING', 'BASILISK_LANDING']):
        logger.info("Test module [LENDING]")

        for app in apps:
            lending_deposit_test_config = {
                'amount': 'all_balance',
                'action': 'DEPOSIT+WITHDRAW',
                'lending_apps': [app],
                'withdraw_sleep_interval': {'from': 0, 'to': 0}
            }

            lending = Lending(zksync_min_native_balance, AccountStorage(), 1)
            lending.execute(*lending.parse_params(lending_deposit_test_config), local_evm_test_account)
