import unittest

from loguru import logger
from sybil_engine.domain.account_storage import AccountStorage
from sybil_engine.utils.utils import ConfigurationException

from web3_wizzard_lib.core.modules import WarmUp
from tests.core.modules.__init__ import zksync_min_native_balance, init_set_test_config, \
    test_min_native_balance, local_evm_test_account
from tests.core.test_config import warmup_allowed_dex

warmup_test_config = {
    'auto_withdrawal': True,
    'chain': 'LINEA',
    'swap_amount_interval': {'from': 1, 'to': 1},
    'token': 'ETH',
    'amount_of_warmups': {'from': 3, 'to': 6},
    'allowed_dex': ['syncswap', 'velocore', 'lineaswap', 'horizondex', 'pancake'],
    # 'allowed_dex': warmup_allowed_dex,
    'warmup_pairs': [],  # specify for warming up by concrete pair
    'sell_tokens': True,
    'pair_sleep_interval': {'from': 0, 'to': 0}  # Interval between different tokens warm up of 1 account
}


class TestWarmUp(unittest.TestCase):
    def test_shouldWarmUp(self):
        init_set_test_config()

        logger.info("Test module [WARMUP, 1]")

        warmup = WarmUp(test_min_native_balance, AccountStorage(), 3)
        warmup_test_config['warmup_pairs'] = []

        try:
            warmup.execute(*warmup.parse_params(warmup_test_config), local_evm_test_account)
        except Exception as e:
            self.fail(f"Some function raised an exception: {e}")

    def test_shouldWarmUpPair(self):
        init_set_test_config()

        logger.info("Test module [WARMUP, 1]")

        warmup = WarmUp(zksync_min_native_balance, AccountStorage(), 3)
        warmup_test_config['warmup_pairs'] = ['ETH>USDC']

        try:
            warmup.execute(*warmup.parse_params(warmup_test_config), local_evm_test_account)
        except Exception as e:
            self.fail(f"Some function raised an exception: {e}")

    def test_shouldWarmUp2Pairs(self):
        logger.info("Test module [WARMUP, 1]")

        warmup_test_config['pair_sleep_interval'] = {'from': 0, 'to': 0}
        warmup_test_config['allowed_dex'] = warmup_allowed_dex
        warmup_test_config['warmup_pairs'] = ['ETH>USDC', 'ETH>USDT']

        warmup = WarmUp(zksync_min_native_balance, AccountStorage(), 2)

        try:
            warmup.execute(*warmup.parse_params(warmup_test_config), local_evm_test_account)
        except Exception as e:
            self.fail(f"Some function raised an exception: {e}")

    def test_shouldSkipNonExistingPair(self):
        init_set_test_config()

        logger.info("Test module [WARMUP, 1]")

        warmup = WarmUp(zksync_min_native_balance, AccountStorage(), 3)
        warmup_test_config['warmup_pairs'] = ['nonExisting']

        with self.assertRaises(ConfigurationException):
            warmup.execute(*warmup.parse_params(warmup_test_config), local_evm_test_account)


