import decimal
import unittest

from loguru import logger
from sybil_engine.data.networks import get_chain_instance_for_network
from sybil_engine.domain.balance.tokens import WETHToken
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.modules import Wrapping
from tests.core.modules.__init__ import  zksync_min_native_balance, init_set_test_config, \
    local_evm_test_account


class TestWrappingModule(unittest.TestCase):
    init_set_test_config()

    web3 = init_web3(get_chain_instance_for_network('SCROLL', 'LOCAL'), None)

    def test_shouldWrapWETH(self, chain='SCROLL', account=local_evm_test_account):
        logger.info("Test module [WRAPPING, 4] WRAP 1 ETH")

        wrap_test_config = {
            'chain': chain,
            'action': 'UNWRAP',  # [WRAP, UNWRAP]
            'amount_interval': {'from': 0, 'to': 0}
        }

        begin_balance = WETHToken(chain, self.web3).balance(account)

        wrapping = Wrapping(zksync_min_native_balance, account)
        wrap_test_config['action'] = 'WRAP'
        wrap_test_config['amount_interval'] = {'from': 1, 'to': 1}

        try:
            wrapping.execute(*wrapping.parse_params(wrap_test_config), account)
        except Exception as e:
            self.fail(f"Some function raised an exception: {e}")

        end_balance = WETHToken(chain, self.web3).balance(account)

        self.assertEqual(end_balance.readable() - begin_balance.readable(), decimal.Decimal(1))

    def test_shouldUnwrapWETH(self, chain='SCROLL', account=local_evm_test_account):
        init_set_test_config()

        logger.info("Test module [WRAPPING, 4] UNWRAP ALL ETH")

        wrap_test_config = {
            'chain': chain,
            'action': 'UNWRAP',  # [WRAP, UNWRAP]
            'amount_interval': {'from': 0, 'to': 0}
        }

        wrapping = Wrapping(zksync_min_native_balance, account)

        wrap_test_config['action'] = 'UNWRAP'
        wrap_test_config['amount_interval'] = 'all_balance'

        try:
            wrapping.execute(*wrapping.parse_params(wrap_test_config), account)
        except Exception as e:
            self.fail(f"Some function raised an exception: {e}")
        end_balance = WETHToken(chain, self.web3).balance(account)

        self.assertEqual(
            end_balance.readable(),
            decimal.Decimal(0)
        )
